"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _BlobDataItem_path, _BlobDataItem_start;
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTemporaryFile = exports.fileFromSync = exports.fileFrom = exports.File = exports.createTemporaryBlob = exports.blobFromSync = exports.blobFrom = exports.Blob = void 0;
const node_fs_1 = require("node:fs");
const node_path_1 = require("node:path");
const node_os_1 = require("node:os");
const node_process_1 = __importDefault(require("node:process"));
const node_domexception_1 = __importDefault(require("node-domexception"));
const file_js_1 = require("./file.js");
Object.defineProperty(exports, "File", { enumerable: true, get: function () { return file_js_1.File; } });
const index_js_1 = require("./index.js");
Object.defineProperty(exports, "Blob", { enumerable: true, get: function () { return index_js_1.Blob; } });
const { stat, mkdtemp } = node_fs_1.promises;
let i = 0, tempDir, registry;
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 */
const blobFromSync = (path, type) => fromBlob((0, node_fs_1.statSync)(path), path, type);
exports.blobFromSync = blobFromSync;
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 * @returns {Promise<Blob>}
 */
const blobFrom = (path, type) => stat(path).then(stat => fromBlob(stat, path, type));
exports.blobFrom = blobFrom;
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 * @returns {Promise<File>}
 */
const fileFrom = (path, type) => stat(path).then(stat => fromFile(stat, path, type));
exports.fileFrom = fileFrom;
/**
 * @param {string} path filepath on the disk
 * @param {string} [type] mimetype to use
 */
const fileFromSync = (path, type) => fromFile((0, node_fs_1.statSync)(path), path, type);
exports.fileFromSync = fileFromSync;
// @ts-ignore
const fromBlob = (stat, path, type = '') => new index_js_1.Blob([new BlobDataItem({
        path,
        size: stat.size,
        lastModified: stat.mtimeMs,
        start: 0
    })], { type });
// @ts-ignore
const fromFile = (stat, path, type = '') => new file_js_1.File([new BlobDataItem({
        path,
        size: stat.size,
        lastModified: stat.mtimeMs,
        start: 0
    })], (0, node_path_1.basename)(path), { type, lastModified: stat.mtimeMs });
/**
 * Creates a temporary blob backed by the filesystem.
 * NOTE: requires node.js v14 or higher to use FinalizationRegistry
 *
 * @param {*} data Same as fs.writeFile data
 * @param {BlobPropertyBag & {signal?: AbortSignal}} options
 * @param {AbortSignal} [signal] in case you wish to cancel the write operation
 * @returns {Promise<Blob>}
 */
const createTemporaryBlob = async (data, { signal, type } = {}) => {
    registry = registry || new FinalizationRegistry(node_fs_1.promises.unlink);
    tempDir = tempDir || await mkdtemp((0, node_fs_1.realpathSync)((0, node_os_1.tmpdir)()) + node_path_1.sep);
    const id = `${i++}`;
    const destination = (0, node_path_1.join)(tempDir, id);
    if (data instanceof ArrayBuffer)
        data = new Uint8Array(data);
    await node_fs_1.promises.writeFile(destination, data, { signal });
    const blob = await blobFrom(destination, type);
    registry.register(blob, destination);
    return blob;
};
exports.createTemporaryBlob = createTemporaryBlob;
/**
 * Creates a temporary File backed by the filesystem.
 * Pretty much the same as constructing a new File(data, name, options)
 *
 * NOTE: requires node.js v14 or higher to use FinalizationRegistry
 * @param {*} data
 * @param {string} name
 * @param {FilePropertyBag & {signal?: AbortSignal}} opts
 * @returns {Promise<File>}
 */
const createTemporaryFile = async (data, name, opts) => {
    const blob = await createTemporaryBlob(data);
    return new file_js_1.File([blob], name, opts);
};
exports.createTemporaryFile = createTemporaryFile;
/**
 * This is a blob backed up by a file on the disk
 * with minium requirement. Its wrapped around a Blob as a blobPart
 * so you have no direct access to this.
 *
 * @private
 */
class BlobDataItem {
    constructor(options) {
        _BlobDataItem_path.set(this, void 0);
        _BlobDataItem_start.set(this, void 0);
        __classPrivateFieldSet(this, _BlobDataItem_path, options.path, "f");
        __classPrivateFieldSet(this, _BlobDataItem_start, options.start, "f");
        this.size = options.size;
        this.lastModified = options.lastModified;
        this.originalSize = options.originalSize === undefined
            ? options.size
            : options.originalSize;
    }
    /**
     * Slicing arguments is first validated and formatted
     * to not be out of range by Blob.prototype.slice
     */
    slice(start, end) {
        return new BlobDataItem({
            path: __classPrivateFieldGet(this, _BlobDataItem_path, "f"),
            lastModified: this.lastModified,
            originalSize: this.originalSize,
            size: end - start,
            start: __classPrivateFieldGet(this, _BlobDataItem_start, "f") + start
        });
    }
    async *stream() {
        const { mtimeMs, size } = await stat(__classPrivateFieldGet(this, _BlobDataItem_path, "f"));
        if (mtimeMs > this.lastModified || this.originalSize !== size) {
            throw new node_domexception_1.default('The requested file could not be read, typically due to permission problems that have occurred after a reference to a file was acquired.', 'NotReadableError');
        }
        yield* (0, node_fs_1.createReadStream)(__classPrivateFieldGet(this, _BlobDataItem_path, "f"), {
            start: __classPrivateFieldGet(this, _BlobDataItem_start, "f"),
            end: __classPrivateFieldGet(this, _BlobDataItem_start, "f") + this.size - 1
        });
    }
    get [(_BlobDataItem_path = new WeakMap(), _BlobDataItem_start = new WeakMap(), Symbol.toStringTag)]() {
        return 'Blob';
    }
}
node_process_1.default.once('exit', () => {
    tempDir && (0, node_fs_1.rmdirSync)(tempDir, { recursive: true });
});
exports.default = blobFromSync;
//# sourceMappingURL=from.js.map